@@ -1,28 +1,33 @@
-# ABSTRACT: A class dispatcher that handles namespaces like paths
+# ABSTRACT: Traverse Class Namspaces
 
 package Class::Forward;
-{
-    $Class::Forward::VERSION = '0.07';
-}
 
 use strict;
 use warnings;
 
-our $VERSION = '0.07';    # VERSION
+our $VERSION = '0.100000'; # VERSION
 
 use Exporter ();
 
 our @ISA    = qw(Exporter);
-our @EXPORT = qw(clsf);
+our @EXPORT = qw(clsf clsr);
 our %CACHE  = ();
 
 
+
 sub clsf {
 
     return Class::Forward->new(namespace => (caller)[0])->forward(@_);
 
 }
 
+
+sub clsr {
+
+    return Class::Forward->new(namespace => (caller)[0])->reverse(@_);
+
+}
+
 sub new {
 
     my $self = bless {}, (shift);
@@ -55,13 +60,14 @@ sub forward {
     my $methods;
     my $myspace;
 
+    my $class   = '';
     my @class   = ();
     my @methods = ();
 
     my $CACHE_KEY = $shorthand;
-    $CACHE_KEY .= "\@$namespace";
+       $CACHE_KEY .= "\@$namespace";
 
-    my $class = $CACHE{$CACHE_KEY} ||= do {
+    my $DATA = $CACHE{$CACHE_KEY} ||= do {
 
         if ($shorthand) {
 
@@ -78,8 +84,8 @@ sub forward {
             # convert shorthand to package notation
 
             $myspace = join "::", map {
-                join '', map { ucfirst lc } split /_/, $_
-            } split /[\-\/]/, $shorthand;
+                /_/ ? join '', map { ucfirst lc } split /_/, $_ : ucfirst $_
+            }   split /(?:::|\-|\/)/, $shorthand;
 
             if ($backspace) {
                 unless ($backspace =~ /^\/$/) {
@@ -137,20 +143,26 @@ sub forward {
         }
 
         # cache the results
-        $CACHE{$CACHE_KEY} = $class;
+        $CACHE{$CACHE_KEY} = {
+            'CLASS'   => $class,
+            'METHODS' => [@methods]
+        };
 
     };
 
+    $class   = $DATA->{'CLASS'};
+    @methods = @{$DATA->{'METHODS'}} if $DATA && ! @methods;
+
     # return result of method call(s) or class name
 
     if (@methods) {
 
-        for (my $i = 0; $i < @methods; $i++) {
+        for (my $i = 0 ; $i < @methods ; $i++) {
 
             my $method = $methods[$i];
 
             $class =
-              $i == $#methods ? $class->$method(@arguments) : $class->$method;
+                $i == $#methods ? $class->$method(@arguments) : $class->$method;
 
         }
 
@@ -166,30 +178,56 @@ sub forward {
 
 }
 
+sub reverse {
+
+    my ($self, $shorthand, $offset, $delimiter) = @_;
+
+    $self->namespace((caller)[0] || 'main') unless $self->namespace;
+
+    $shorthand =~ s/((\.\w+){1,})$// if $shorthand;
+    $delimiter ||= '/';
+
+    my $result = $self->forward($shorthand);
+    my @pieces = split /::/, $result;
+
+    if (defined $offset and $offset >= 0) {
+        if ($offset == 0) {
+            unshift @pieces, '';
+        }
+        else {
+            shift @pieces for (1..$offset);
+        }
+    }
+    else {
+        unshift @pieces, '';
+    }
+
+    return join $delimiter,
+        map { if ($_) { s/([a-z])([A-Z])/$1_\l$2/g; lc } } @pieces;
+
+}
+
 
 1;
 
 __END__
+
 =pod
 
 =head1 NAME
 
-Class::Forward - A class dispatcher that handles namespaces like paths
+Class::Forward - Traverse Class Namspaces
 
 =head1 VERSION
 
-version 0.07
+version 0.100000
 
 =head1 SYNOPSIS
 
-    # OO Usage and Syntax
-
     package MyApp;
 
     use Class::Forward;
 
-    ...
-
     sub class {
 
         my ($self, $shorthand, @arguments) = @_;
@@ -202,22 +240,21 @@ version 0.07
 
     package main;
 
-    my $app = MyApp->new;
-
+    my $app  = MyApp->new;
     my $data = $app->class('data.new'); # returns a new MyApp::Data object
 
 =head1 DESCRIPTION
 
-Class::Forward is designed to simply return class names or dispatch method calls
-using shorthand. It uses file-system path specification conventions to match
-against class namespaces.
+Class::Forward is designed to simply return class names and/or dispatch method
+calls using shorthand. It uses file-system path specification conventions to
+match against class namespaces.
 
 =head1 EXPORTS
 
 =head2 clsf
 
 The exported function clsf is responsible for resolving your shorthand. It
-begets the follow functionality:
+provides the follow functionality:
 
     package App::Store;
 
@@ -236,8 +273,7 @@ begets the follow functionality:
     clsf '//view.new.render';         # ... dispatches methods in succession
     clsf 'cgi';                       # returns App::Store::Cgi
     clsf '/cgi';                      # returns Cgi (or CGI if already loaded)
-
-    # yada yada
+                                      # ... it tries to do the right thing
 
     1;
 
@@ -245,17 +281,39 @@ The clsf function takes two arguments, the shorthand to be translated, and an
 optional list of arguments to be passed to the last method appended to the
 shorthand.
 
-NOTE: Class::Forward also has support for walking up a path although this should
-be done with caution, writing code that executes other code based on its
-namespace's relativity to the current class could yield unpredictable results,
-especially if the calling class is ever relocated.
+=head2 clsr
+
+The exported function clsr is responsible for resolving your shorthand. It
+provides the follow functionality:
+
+    package App::Store;
+
+    use CGI;
+    use Class::Forward;
+
+    clsr;                             # returns /app/store
+    clsr './user';                    # returns /app/store/user
+    clsr './user.new', name => 'N30'; # returns /app/store/user
+    clsr './user_profile';            # returns /app/store/user_profile
+    clsr '../user';                   # returns /app/user
+    clsr '//';                        # returns /app
+    clsr '//.new';                    # returns /app
+    clsr '//view';                    # returns /app/view
+    clsr '//view.new';                # returns /app/view
+    clsr '//view.new.render';         # returns /app/view
+    clsr 'cgi';                       # returns /app/store/cgi
+    clsr '/cgi';                      # returns /cgi
+
+    1;
+
+The clsr function takes two arguments, the shorthand to be translated, and an
+optional list of arguments to be passed to the last method appended to the
+shorthand.
 
 =head1 SEE ALSO
 
 Along my travels I recall visiting a similar module on the CPAN called
 L<Namespace::Dispatch> which provides somewhat of the same functionality.
-Additionally, since this module looks like something INGY would upload, you
-might want to check out his offerings also.
 
 =head1 AUTHOR
 
@@ -269,4 +327,3 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
